const KeyAgreementNtor = require('./KeyAgreementNtor');
const CircuitNodeCryptoState = require('./CircuitNodeCryptoState');

class CircuitNode {
  _circuit = null
  /** @type OnionRouter */
  _onion_router = null;
  _type = 'normal';
  /** @type KeyAgreementNtor */
  _handshake = null;
  /** @type CircuitNodeCryptoState */
  _crypto_state = null;

  get onion_router() {
    return this._onion_router;
  }

  constructor (circuit, or, type) {
    this._circuit = circuit;
    this._onion_router = or;
    this._type = type;
  }

  create_onion_skin_ntor() {
    this._handshake = new KeyAgreementNtor(this._onion_router);

    return Buffer.concat([
      this._onion_router.identity_fingerprint,
      this._onion_router.ntor_onion_key,
      this._handshake.public_key
    ], 20 + 32 + 32);
  }

  compute_shared_secret(handshake_data) {
    const key_material = this._handshake.compute_shared_secret(handshake_data);

    if (key_material)
    {
      //
      // struct key_material
      // {
      //   byte_type digest_forward [20];
      //   byte_type digest_backward[20];
      //   byte_type cipher_forward [16];
      //   byte_type cipher_backward[16];
      //   ^^ sizeof == 40 + 32 == 72 ^^
      //
      //   byte_type rend_nonce     [20]; << ignored now
      //   ^^ sizeof == 72 + 20 == 92 ^^  << (used in establishing of introduction points,
      //                                      rend-spec.txt � 1.2.)
      //
      //   byte_type __garbage__    [];   << ignored
      // };
      //

      this._crypto_state = new CircuitNodeCryptoState(key_material);
    }
  }

  has_valid_crypto_state() {
    return this._crypto_state != null;
  }

  /**
   * @param {RelayCell} cell
   */
  encrypt_forward_cell(cell) {
    return this._crypto_state.encrypt_forward_cell(cell);
  }

  /**
   * @param {Cell} cell
   * @returns {boolean}
   */
  decrypt_backward_cell(cell) {
    return this._crypto_state.decrypt_backward_cell(cell);
  }

  decrement_deliver_window() {}
  consider_sending_sendme() {}
  increment_package_window() {}
  decrement_package_window() {}
}
module.exports = CircuitNode;
