class OnionRouter {
  static none            = 0x0000;
  static authority       = 0x0001;
  static bad_exit        = 0x0002;
  static exit            = 0x0004;
  static fast            = 0x0008;
  static guard           = 0x0010;
  static hsdir           = 0x0020;
  static named           = 0x0040;
  static no_ed_consensus = 0x0080;
  static stable          = 0x0100;
  static running         = 0x0200;
  static unnamed         = 0x0400;
  static valid           = 0x0800;
  static v2dir           = 0x1000;

  /** @type Consensus */
  _consensus = null;
  _nickname = '';
  _ip = '';
  _or_port = 0;
  _dir_port = 0;
  /** @type Buffer */
  _identity_fingerprint = null;
  _descriptor_fetched = false;
  _ntor_onion_key = null;
  _service_key = null;

  constructor (consensus, nickname, ip, or_port, dir_port, identity_fingerprint) {
    this._consensus = consensus;
    this._nickname = nickname;
    this._ip = ip;
    this._or_port = or_port;
    this._dir_port = dir_port;
    this._identity_fingerprint = identity_fingerprint;
  }

  get consensus() { return this._consensus; }
  get name() { return this._nickname; }
  get ip() { return this._ip; }
  get flags() { return this._flags; }
  get or_port() { return this._or_port; }
  get dir_port() { return this._dir_port; }
  set flags(flags) { this._flags = flags; }
  get ntor_onion_key() {
    if (!this._descriptor_fetched) throw new Error('forgot to call fetch_descriptor for onion router');
    return this._ntor_onion_key;
  }
  get identity_fingerprint() { return this._identity_fingerprint; }
  set service_key(key) { this._service_key = key; }
  get service_key() { return this._service_key; }

  async fetch_descriptor() {
    const descriptor = await this._consensus.get_onion_router_descriptor(this._identity_fingerprint);
    // parse
    const lines = descriptor.split('\n');
    for (const line of lines){
      const parts = line.split(' ');
      if (parts[0] === 'ntor-onion-key') {
        this._ntor_onion_key = Buffer.from(parts[1], 'base64');
        break;
      }
    }
    this._descriptor_fetched = true;
  }
}

module.exports = OnionRouter;
