const http = require('http');
const https = require('https');
const tls = require('tls');

/**
 * @param {string} protocol
 * @param {string} ip
 * @param {string} port
 * @param {string} path
 * @param {object} [stream]
 * @return {Promise<string>}
 */
async function get(protocol, ip, port, path, stream) {
  let options = {};
  if (stream) {
    options.createConnection = function(args, cb) {
      if (protocol === 'https:') {
        return new tls.connect({socket: stream, servername: args.hostname}, () => cb());
      }
      return stream;
    }
  }

  const proto = protocol === 'https:' ? https : http;
  return new Promise((resolve, reject) => {
    proto.get(`${protocol}//${ip}:${port}${path}`, options, res => {
      let data = '';
      res.on('data', chunk => data += chunk.toString());
      res.on('end', () => resolve(data));
      res.on('error', (err) => reject(err));
    });
  });
}

exports.get = get;
